# -*- coding: utf-8 -*-

#changer le dossier par défaut
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/boosting_randomforest")

#importation des données
import pandas
image_all = pandas.read_table("image.txt",sep="\t",header=0,decimal=".")

#dimension des données
print(image_all.shape)

#nom de variables
print(image_all.columns)

#type des données
print(image_all.dtypes)

#afficher les 5 premières obs.
print(image_all.head())

#5 premières lignes et les 1ere et dernière colonnes
print(image_all.iloc[:5,[0,20]])

#séparer en apprentissage et test

#apprentissage - sélectionner les lignes
image_train = image_all[image_all["sample"]=="train"]
#retirer la colonne sample
image_train = image_train.iloc[:,0:20]
#vérification
print(image_train.shape)
print(image_train.head())
#transformation en matrices numpy
d_train = image_train.as_matrix()
y_app = d_train[:,0]
X_app = d_train[:,1:20]

#test
image_test = (image_all[image_all["sample"]=="test"]).iloc[:,0:20]
print(image_test.shape)
y_test = image_test.as_matrix()[:,0]
X_test = image_test.as_matrix()[:,1:20]

#module pour évaluation des classifieurs
from sklearn import metrics

#fonction pour évaluation de méthodes
def error_rate(modele,y_test,X_test):
    #prediction
    y_pred = modele.predict(X_test)
    #taux d'erreur
    err = 1.0 - metrics.accuracy_score(y_test,y_pred)
    #return
    return err
#fin fonction
    
#arbre de décision
from sklearn.tree import DecisionTreeClassifier

#instanciation
dtree = DecisionTreeClassifier()

#paramètres de l'arbre
print(dtree)

#apprentissage
dtree.fit(X_app,y_app)

#***affichage de l'arbre

#génération de la sortie au format .dot
from sklearn import tree
tree.export_graphviz(dtree,out_file="tree.dot",feature_names=image_train.columns[1:20],class_names=image_train.columns[0])


#importance des variables - 0 quand elle n'apparaît pas
imp = {"VarName":image_train.columns[1:],"Importance":dtree.feature_importances_}
print(pandas.DataFrame(imp))

#taux d'erreur
print(error_rate(dtree,y_test,X_test))

#classe bagging
from sklearn.ensemble import BaggingClassifier

#instanciation
baggingTree = BaggingClassifier(DecisionTreeClassifier(),n_estimators=20)
print(baggingTree)

#apprentissage
baggingTree.fit(X_app,y_app)

#évaluation
print(error_rate(baggingTree,y_test,X_test))

#expérimentation - nombre d'arbres

#fonction train-test pour un m donné
def train_test_bagging(m,X_app,y_app,X_test,y_test):
    #modélisation
    bag = BaggingClassifier(DecisionTreeClassifier(),n_estimators=m)
    bag.fit(X_app,y_app)
    return error_rate(bag,y_test,X_test)
#fin train-test

#valeurs de m à tester
m_a_tester = [1,5,10,20,50,100,200]

#matrice de résultat
import numpy
result = numpy.zeros(shape=(1,7))

#répéter 20 fois l'expérience
for expe in range(20):
    #itérer sur ces valeurs
    res = [train_test_bagging(m,X_app,y_app,X_test,y_test) for m in m_a_tester]
    res = numpy.asarray(res).reshape(1,7)
    #ajouter comme nouvelle ligne dans la matrice
    result = numpy.append(result,res,axis=0)
#

#retirer la première ligne de zéros
result = numpy.delete(result,0,axis=0)

#calculer les moyennes
mresult = numpy.mean(result,axis=0)
print(mresult)

#graphique
import matplotlib.pyplot as plt

#label des axes
plt.xlabel("m")
plt.ylabel("Err. Rate")

plt.plot(m_a_tester,mresult,linewidth=2)


#recherche du nombre "optimal" d'arbres
#outil grille de recherche
from sklearn.grid_search import GridSearchCV

#paramètres à manipuler
#le nom du paramère à manipuler est explicite
#on énumère alors les valeurs à essayer
parametres = [{"n_estimators":[1,5,10,20,50,100,200]}]

#instanciation du classifieur à inspecter
bag = BaggingClassifier(DecisionTreeClassifier())

#instanciation
grid_bag = GridSearchCV(estimator=bag,param_grid=parametres,scoring="accuracy")

#lancer l'exploration
grille_bag = grid_bag.fit(X_app,y_app)

#affichage des résultats
print(grille_bag.grid_scores_)

#meilleur score
print(grille_bag.best_score_)

#meilleur paramétrage
print(grille_bag.best_params_)

#nouvelle évaluation
print(error_rate(grille_bag,y_test,X_test))

#analyse discriminante
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis

#instanciation
bag_lda = BaggingClassifier(LinearDiscriminantAnalysis(),n_estimators=20)
print(bag_lda)

#apprentissage
bag_lda.fit(X_app,y_app)

#évaluation
print(error_rate(bag_lda,y_test,X_test))

#analyse discriminante seule
lda = LinearDiscriminantAnalysis()
lda.fit(X_app,y_app)
print(error_rate(lda,y_test,X_test))


#classe RandomForest
from sklearn.ensemble import RandomForestClassifier

#instanciation
rf = RandomForestClassifier(n_estimators=20)

#apprentissage
rf.fit(X_app,y_app)

#taux d'erreur
print(error_rate(rf,y_test,X_test))

#importance des variables
print(rf.feature_importances_)

#avec les noms des variables
imp = {"VarName":image_train.columns[1:],"Importance":rf.feature_importances_}
print(pandas.DataFrame(imp))

#recherche du nombre "optimal" d'arbres
from sklearn.grid_search import GridSearchCV

#paramètres à tester
parametres = [{"n_estimators":[1,5,10,20,50,100,200]}]

#instanciation
grid_rf = GridSearchCV(estimator=rf,param_grid=parametres,scoring="accuracy")

#lancer l'exploration
grille_rf = grid_rf.fit(X_app,y_app)

#affichage des résultats
print(grille_rf.grid_scores_)

#meilleur score
print(grille_rf.best_score_)

#meilleur paramétrage
print(grille_rf.best_params_)

#nouvelle évaluation
print(error_rate(grille_rf,y_test,X_test))

#Adaboost
from sklearn.ensemble import AdaBoostClassifier

#instanciation
ab = AdaBoostClassifier(algorithm="SAMME",n_estimators=20, base_estimator = DecisionTreeClassifier())
print(ab)

#apprentissage
ab.fit(X_app,y_app)

#evaluation
print(error_rate(ab,y_test,X_test))
